@@ -1,7 +1,7 @@
 """
 ///////////////////////////////////////////////////////////////////////////////
 //        Copyright (c) 2012-2021 Oscar Riveros. all rights reserved.        //
-//                        oscar.riveros@peqnp.science                        //
+//                        oscar.riveros@satx.science                        //
 //                                                                           //
 //   without any restriction, Oscar Riveros reserved rights, patents and     //
 //  commercialization of this knowledge or derived directly from this work.  //
@@ -23,10 +23,10 @@ SOFTWARE.
 The standard high level library for the SAT-X system.
 """
 
+from .alu import *
 from .gaussian import Gaussian
 from .linear import Linear
 from .rational import Rational
-from .alu import *
 
 csp = None
 render = False
@@ -38,7 +38,7 @@ def version():
     """
     print('SAT-X Mathematical Solver from http://www.peqnp.com')
     print('Copyright (c) 2012-2021 Oscar Riveros. all rights reserved.')
-    print('oscar.riveros@peqnp.science')
+    print('oscar.riveros@satx.science')
 
 
 def check_engine():
@@ -84,6 +84,18 @@ def integer(bits=None):
     return csp.variables[-1]
 
 
+def natural(bits=None):
+    """
+    Correspond to an unsigned integer.
+    :param bits: The bits for the unsigned integer.
+    :return: An instance of Unsigned Integer.
+    """
+    global csp
+    check_engine()
+    csp.variables.append(csp.nat(size=bits))
+    return csp.variables[-1]
+
+
 def constant(value, bits=None):
     """
     Correspond to an constant.
@@ -97,14 +109,14 @@ def constant(value, bits=None):
     return csp.variables[-1]
 
 
-def satisfy(turbo=False, log=False):
+def satisfy(solve=True, turbo=False, log=False, cnf_path=''):
     """
     Find a model for the current problem.
     :param turbo: This make a simplification of the model, is more fast to solve, but destroy the internal structure of the problem, need regenerate, use for only one solution.
     :param log: Show log for the SLIME SAT Solver.
     :return: True if SATISFIABLE else False
     """
-    return csp.to_sat(solve=True, turbo=turbo, log=log, assumptions=None, cnf_path='', model_path='', proof_path='')
+    return csp.to_sat(solve=solve, turbo=turbo, log=log, assumptions=None, cnf_path=cnf_path, model_path='', proof_path='')
 
 
 def subsets(lst, k=None, complement=False):
@@ -117,7 +129,7 @@ def subsets(lst, k=None, complement=False):
     """
     global csp
     check_engine()
-    bits = csp.int(size=len(lst))
+    bits = csp.nat(size=len(lst))
     csp.variables.append(bits)
     if k is not None:
         assert sum(csp.zero.iff(-bits[i], csp.one) for i in range(len(lst))) == k
@@ -153,11 +165,12 @@ def subset(lst, k, empty=None, complement=False):
     return subset_
 
 
-def vector(bits=None, size=None, is_gaussian=False, is_rational=False, is_mip=False, is_real=False):
+def vector(bits=None, size=None, signed=True, is_gaussian=False, is_rational=False, is_mip=False, is_real=False):
     """
     A vector of integers.
     :param bits: The bit bits for each integer.
     :param size: The bits of the vector.
+    :param signed: integer or natural components.
     :param is_gaussian: Indicate of is a Gaussian Integers vector.
     :param is_rational: Indicate of is a Rational vector.
     :param is_mip: Indicate of is a MIP vector.
@@ -167,25 +180,26 @@ def vector(bits=None, size=None, is_gaussian=False, is_rational=False, is_mip=Fa
     global csp
     check_engine()
     if is_rational:
-        return [rational() for _ in range(size)]
+        return [rational(signed=signed) for _ in range(size)]
     if is_gaussian:
-        return [gaussian() for _ in range(size)]
+        return [gaussian(signed=signed) for _ in range(size)]
     if is_mip:
         lns = []
         for _ in range(size):
             lns.append(linear(is_real=is_real))
         return lns
     else:
-        array_ = csp.array(size=bits, dimension=size)
+        array_ = csp.array(signed=signed, size=bits, dimension=size)
         csp.variables += array_
     return array_
 
 
-def matrix(bits=None, dimensions=None, is_gaussian=False, is_rational=False, is_mip=False, is_real=False):
+def matrix(bits=None, dimensions=None, signed=True, is_gaussian=False, is_rational=False, is_mip=False, is_real=False):
     """
     A matrix of integers.
     :param bits: The bit bits for each integer.
     :param dimensions: An tuple with the dimensions for the Matrix (n, m).
+    :param signed: integer or natural components.
     :param is_gaussian: Indicate of is a Gaussian Integers vector.
     :param is_rational: Indicate of is a Rational Matrix.
     :param is_mip: Indicate of is a MIP Matrix.
@@ -204,15 +218,30 @@ def matrix(bits=None, dimensions=None, is_gaussian=False, is_rational=False, is_
                 row.append(lns[-1])
             else:
                 if is_rational:
-                    x = integer(bits=bits)
-                    y = integer(bits=bits)
-                    csp.variables.append(rational(x, y))
+                    if signed:
+                        x = integer(bits=bits)
+                        y = integer(bits=bits)
+                    else:
+                        x = natural(bits=bits)
+                        y = natural(bits=bits)
+                    csp.variables.append(x)
+                    csp.variables.append(y)
+                    row.append(Rational(x, y))
                 elif is_gaussian:
-                    x = integer(bits=bits)
-                    y = integer(bits=bits)
-                    csp.variables.append(gaussian(x, y))
+                    if signed:
+                        x = integer(bits=bits)
+                        y = integer(bits=bits)
+                    else:
+                        x = natural(bits=bits)
+                        y = natural(bits=bits)
+                    csp.variables.append(x)
+                    csp.variables.append(y)
+                    row.append(Gaussian(x, y))
                 else:
-                    csp.variables.append(integer(bits=bits))
+                    if signed:
+                        csp.variables.append(integer(bits=bits))
+                    else:
+                        csp.variables.append(natural(bits=bits))
                 row.append(csp.variables[-1])
         matrix_.append(row)
     return matrix_
@@ -520,29 +549,39 @@ def index(ith, data):
     return csp.variables[-1]
 
 
-def gaussian(x=None, y=None):
+def gaussian(x=None, y=None, signed=True):
     """
     Create a gaussian integer from (x+yj).
     :param x: real
     :param y: imaginary
+    :param signed: integer or natural components, if x and y not provided.
     :return: (x+yj)
     """
     check_engine()
-    if x is None and y is None:
-        return Gaussian(integer(), integer())
+    if signed:
+        if x is None and y is None:
+            return Gaussian(integer(), integer())
+    else:
+        if x is None and y is None:
+            return Gaussian(natural(), natural())
     return Gaussian(x, y)
 
 
-def rational(x=None, y=None):
+def rational(x=None, y=None, signed=True):
     """
     Create a rational x / y.
     :param x: numerator
     :param y: denominator
+    :param signed: integer or natural components, if x and y not provided.
     :return: x / y
     """
     check_engine()
-    if x is None and y is None:
-        return Rational(integer(), integer())
+    if signed:
+        if x is None and y is None:
+            return Rational(integer(), integer())
+    else:
+        if x is None and y is None:
+            return Rational(natural(), natural())
     return Rational(x, y)
 
 
@@ -571,7 +610,7 @@ def sqrt(x):
 
 # ///////////////////////////////////////////////////////////////////////////////
 # //        Copyright (c) 2012-2020 Oscar Riveros. all rights reserved.        //
-# //                        oscar.riveros@peqnp.science                        //
+# //                        oscar.riveros@satx.science                        //
 # //                                                                           //
 # //   without any restriction, Oscar Riveros reserved rights, patents and     //
 # //  commercialization of this knowledge or derived directly from this work.  //
@@ -642,7 +681,7 @@ def hess_sequence(n, oracle, fast=False, cycles=1, target=0):
 
 # ///////////////////////////////////////////////////////////////////////////////
 # //        Copyright (c) 2012-2020 Oscar Riveros. all rights reserved.        //
-# //                        oscar.riveros@peqnp.science                        //
+# //                        oscar.riveros@satx.science                        //
 # //                                                                           //
 # //   without any restriction, Oscar Riveros reserved rights, patents and     //
 # //  commercialization of this knowledge or derived directly from this work.  //
@@ -713,7 +752,7 @@ def hess_binary(n, oracle, fast=False, cycles=1, target=0):
 
 # ///////////////////////////////////////////////////////////////////////////////
 # //        Copyright (c) 2012-2020 Oscar Riveros. all rights reserved.        //
-# //                        oscar.riveros@peqnp.science                        //
+# //                        oscar.riveros@satx.science                        //
 # //                                                                           //
 # //   without any restriction, Oscar Riveros reserved rights, patents and     //
 # //  commercialization of this knowledge or derived directly from this work.  //
@@ -782,7 +821,7 @@ def hess_abstract(xs, oracle, f, g, log=None, fast=False, cycles=1, target=0):
 
 # ///////////////////////////////////////////////////////////////////////////////
 # //        Copyright (c) 2012-2020 Oscar Riveros. all rights reserved.        //
-# //                        oscar.riveros@peqnp.science                        //
+# //                        oscar.riveros@satx.science                        //
 # //                                                                           //
 # //   without any restriction, Oscar Riveros reserved rights, patents and     //
 # //  commercialization of this knowledge or derived directly from this work.  //
@@ -824,7 +863,7 @@ def tensor(dimensions):
     """
     global csp
     check_engine()
-    csp.variables.append(csp.int(size=None, deep=dimensions))
+    csp.variables.append(csp.nat(size=None, deep=dimensions))
     return csp.variables[-1]
 
 
@@ -949,12 +988,19 @@ def external_satisfy(solver, params=''):
     check_engine()
     if csp.cnf == '':
         raise Exception('CNF path not set.')
+    try:
+        key = csp.cnf[:csp.cnf.index('.')]
+    except Exception as ex:
+        print('No .cnf extension found. {}'.format(ex))
     if not render:
         render = True
-        csp.cnf_file.close()
+        csp.cnf_file = open(csp.cnf, 'r+')
+        header = 'p cnf {} {}'.format(csp.number_of_variables, csp.number_of_clauses)
+        content = csp.cnf_file.read()
+        csp.cnf_file.seek(0, 0)
+        csp.cnf_file.write(header.rstrip('\r\n') + '\n' + content)
     if '.' not in csp.cnf:
         raise Exception('CNF has no extension.')
-    key = csp.cnf[:csp.cnf.index('.')]
     subprocess.call('{0} {2} {1}.cnf > {1}.mod'.format(solver, key, params), shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
     with open('{}.mod'.format(key), 'r') as mod:
         lines = ''
@@ -965,15 +1011,15 @@ def external_satisfy(solver, params=''):
             model = list(map(int, lines.strip(' ').split(' ')))
             for key, value in csp.map.items():
                 for arg in csp.variables:
-                    if isinstance(arg, Unit) and arg.key == key:
-                        ds = ''.join(map(str, [int(int(model[abs(bit) - 1]) > 0) for bit in value[::-1]]))
+                    ds = ''.join(map(str, [int(int(model[abs(bit) - 1]) > 0) for bit in value[::-1]]))
+                    if arg.signed and arg.key == key:
                         if ds[0] == '1':
                             arg.value = -int(''.join(['0' if d == '1' else '1' for d in ds[1:]]), 2) - 1
                         else:
                             arg.value = int(ds[1:], 2)
                         del arg.bin[:]
-            if not csp.cnf_file.closed:
-                csp.cnf_file.close()
+                    if not arg.signed and arg.key == key:
+                        arg.value = int(''.join(map(str, [int(int(model[abs(bit) - 1]) > 0) for bit in value[::-1]])), 2)
             with open(csp.cnf, 'a') as file:
                 file.write(' '.join([str(-int(literal)) for literal in model]) + '\n')
             return True
